@@ -9,14 +9,16 @@ from ansys.fluent.core._version import __version__  # noqa: F401
 from ansys.fluent.core.launcher.launcher import (  # noqa: F401
     FluentVersion,
     LaunchModes,
+    get_ansys_version,
     launch_fluent,
     set_ansys_version,
     set_fluent_path,
 )
 from ansys.fluent.core.session import _BaseSession as Fluent  # noqa: F401
 from ansys.fluent.core.utils.logging import LOG
+from ansys.fluent.core.utils.setup_for_fluent import setup_for_fluent  # noqa: F401
 
-_VERSION_INFO = "Build date: September 02, 2022 17:31 UTC ShaID: ed40716"
+_VERSION_INFO = "Build date: January 17, 2023 15:37 UTC ShaID: 4af0df1"
 """Global variable indicating the version of the PyFluent package - Empty by default"""
 
 _THIS_DIRNAME = os.path.dirname(__file__)
