@@ -1,9 +1,10 @@
-"""Wrappers over TUI-based datamodel grpc service of Fluent."""
+"""Wrappers over TUI-based datamodel gRPC service of Fluent."""
 
 import keyword
 import types
-from typing import Any, Iterable, List, Tuple, Union
+from typing import Any, Dict, Iterable, List, Tuple, Union
 
+from google.protobuf.json_format import MessageToDict
 import grpc
 
 from ansys.api.fluent.v0 import datamodel_tui_pb2 as DataModelProtoModule
@@ -16,9 +17,9 @@ Path = List[str]
 
 
 class DatamodelService:
-    """Class wrapping the TUI-based datamodel grpc service of Fluent.
+    """Class wrapping the TUI-based datamodel gRPC service of Fluent.
 
-    It is suggested to use the methods from PyMenu class.
+    Using the methods from PyMenu class is recommended.
     """
 
     def __init__(self, channel: grpc.Channel, metadata: List[Tuple[str, str]]):
@@ -57,6 +58,10 @@ class DatamodelService:
     ) -> DataModelProtoModule.ExecuteQueryResponse:
         return self.__stub.ExecuteQuery(request, metadata=self.__metadata)
 
+    @catch_grpc_error
+    def get_static_info(self, request):
+        return self.__stub.GetStaticInfo(request, metadata=self.__metadata)
+
 
 def _convert_value_to_gvalue(val: Any, gval: Variant):
     """Convert Python datatype to Value type of
@@ -102,17 +107,17 @@ def _convert_gvalue_to_value(gval: Variant):
 
 class PyMenu:
     """Pythonic wrapper of TUI-based DatamodelService class. Use this class
-    instead of directly calling DatamodelService's method.
+    instead of directly calling the DatamodelService's method.
 
     Methods
     -------
     get_child_names(include_unavailable):
-        Get child menu names
+        Get child menu names.
     execute(*args, **kwargs)
-        Execute command/query at menu with positional or keyword
-        arguments
+        Execute a command or query at a menu with positional or keyword
+        arguments.
     get_doc_string(include_unavailable)
-        Get docstring for menu
+        Get docstring for a menu.
     """
 
     def __init__(self, service: DatamodelService, path: Union[Path, str]):
@@ -120,17 +125,17 @@ class PyMenu:
         self._path = path if isinstance(path, str) else convert_path_to_grpc_path(path)
 
     def get_child_names(self, include_unavailable: bool = False) -> List[str]:
-        """Get child menu names.
+        """Get the names of child menus.
 
         Parameters
         ----------
         include_unavailable : bool, optional
-            When True, will query over static TUI metadata,
-            by default False
+            Whether to query over static TUI metadata. The default is ``False``.
 
         Returns
         -------
         List[str]
+            Names of child menus.
         """
         request = DataModelProtoModule.GetAttributeValueRequest()
         request.path = self._path
@@ -153,7 +158,8 @@ class PyMenu:
         return _convert_gvalue_to_value(ret.result)
 
     def execute(self, *args, **kwargs) -> Any:
-        """Execute command/query at path with positional or keyword arguments.
+        """Execute a command or query at a path with positional or keyword
+        arguments.
 
         Parameters
         ----------
@@ -162,7 +168,7 @@ class PyMenu:
         -------
         Any
             Query result (any Python datatype) or Future object
-            wrapping TUI output of a command
+            wrapping TUI output of a command.
         """
         request = DataModelProtoModule.ExecuteCommandRequest()
         request.path = self._path
@@ -177,13 +183,12 @@ class PyMenu:
             return self._execute_command(request)
 
     def get_doc_string(self, include_unavailable: bool = False) -> str:
-        """Get docstring for menu.
+        """Get docstring for a menu.
 
         Parameters
         ----------
         include_unavailable : bool, optional
-            When True, will query over static TUI metadata,
-            by default False
+            Whether to query over static TUI metadata. The default is ``False``.
 
         Returns
         -------
@@ -197,6 +202,45 @@ class PyMenu:
         response = self._service.get_attribute_value(request)
         return _convert_gvalue_to_value(response.value)
 
+    def get_static_info(self) -> Dict[str, Any]:
+        """Get static info at menu level.
+
+        Returns
+        -------
+        DataModelProtoModule.StaticInfo
+            static info
+        """
+        try:
+            request = DataModelProtoModule.GetStaticInfoRequest()
+            request.path = self._path
+            response = self._service.get_static_info(request)
+            return MessageToDict(response.info, including_default_value_fields=True)
+        except RuntimeError:
+            return _get_static_info_at_level(self)
+
+
+def _get_static_info_at_level(menu: PyMenu) -> Dict[str, Any]:
+    info = {}
+    info["help"] = menu.get_doc_string(include_unavailable=True)
+    info["menus"] = {}
+    info["commands"] = {}
+    child_names = menu.get_child_names(include_unavailable=True)
+    if child_names:
+        for child_name in child_names:
+            if child_name:
+                child_menu = PyMenu(
+                    menu._service,
+                    menu._path + ("" if menu._path.endswith("/") else "/") + child_name,
+                )
+                child_info = _get_static_info_at_level(child_menu)
+                if child_info.pop("is_command", False):
+                    info["commands"][child_name] = child_info
+                else:
+                    info["menus"][child_name] = child_info
+    else:
+        info["is_command"] = True
+    return info
+
 
 class TUIMenu:
     """Base class for the generated menu classes."""
@@ -212,31 +256,26 @@ class TUIMenu:
         ]
 
     def __getattribute__(self, name):
-        attr = super().__getattribute__(name)
-        if type(attr) == types.MethodType:
-            path = self.path + [name]
-            # if menus are inserted at runtime in Fluent
-            if PyMenu(self.service, path).get_child_names():
-                return TUIMenuGeneric(path, self.service)
-        return attr
+        try:
+            attr = super().__getattribute__(name)
+            if type(attr) == types.MethodType:
+                # some runtime submenus are generated as methods during codegen
+                path = self.path + [name]
+                if PyMenu(self.service, path).get_child_names():
+                    return TUIMenu(path, self.service)
+            return attr
+        except AttributeError as ex:
+            if name in dir(self):
+                # for runtime submenus and commands which are not available during codegen
+                path = self.path + [name]
+                if PyMenu(self.service, path).get_child_names():
+                    return TUIMenu(path, self.service)
+                else:
+                    return TUICommand(path, self.service)
+            else:
+                raise ex
 
-
-class TUIMenuGeneric(TUIMenu):
-    """Generic menu class for when the explicit menu classes aren't
-    available."""
-
-    def __getattribute__(self, name):
-        if name in ["path", "service"]:
-            return super().__getattribute__(name)
-        name = convert_func_name_to_tui_menu(name)
-        path = self.path + [name]
-        if PyMenu(self.service, path).get_child_names():
-            return TUIMenuGeneric(path, self.service)
-        else:
-            return TUICommandGeneric(path, self.service)
-
-
-class TUICommandGeneric(TUIMenu):
+class TUICommand(TUIMenu):
     """Generic command class for when the explicit menu classes aren't
     available."""
 
@@ -245,11 +284,12 @@ class TUICommandGeneric(TUIMenu):
 
 
 def convert_func_name_to_tui_menu(func_name: str) -> str:
-    """Convert Python function name to TUI menu string.
+    """Convert a Python function name to a TUI menu string.
 
     Parameters
     ----------
     func_name : str
+       Name of the Python function.
 
     Returns
     -------
@@ -261,11 +301,12 @@ def convert_func_name_to_tui_menu(func_name: str) -> str:
 
 
 def convert_tui_menu_to_func_name(menu: str) -> str:
-    """Convert TUI menu string to Python function name.
+    """Convert a TUI menu string to a Python function name.
 
     Parameters
     ----------
     menu : str
+       TUI menu string.
 
     Returns
     -------
@@ -280,17 +321,17 @@ def convert_tui_menu_to_func_name(menu: str) -> str:
 
 
 def convert_path_to_grpc_path(path: Path) -> str:
-    """Convert path structure to a string which can be passed to datamodel grpc
-    service.
+    """Convert a path structure to a string that can be passed to the data
+    model gRPC service.
 
     Parameters
     ----------
     path : Path
-        Path structure
+        Path structure.
 
     Returns
     -------
     str
-        grpc path
+        gRPC path.
     """
     return "/" + "/".join(convert_func_name_to_tui_menu(x) for x in path)
